# $Id: AxKit.pm,v 1.29 2000/08/06 16:51:36 matt Exp $

package AxKit;
use strict;
use vars qw/$VERSION/;

use DynaLoader ();
use UNIVERSAL ();
use Apache;
use Apache::Log;
use Apache::Constants;
use Apache::ModuleConfig ();
use Apache::AxKit::Provider;
use Apache::AxKit::Exception;
use Apache::AxKit::ConfigReader;
use Apache::AxKit::Cache;
use Apache::AxKit::Provider;
use Apache::AxKit::StyleProvider;
use Apache::AxKit::Provider::File;

$VERSION = "0.98";

if ($ENV{MOD_PERL}) {
	no strict;
	@ISA = qw(DynaLoader);
	__PACKAGE__->bootstrap($VERSION);
}

###########################################################
# Configuration Directive Callbacks
###########################################################

sub AxResetStyleMap ($$) {
	my ($cfg, $parms) = @_;
	%{$cfg->{StyleMap}} = ();
}

sub AxAddStyleMap ($$$$) {
	my ($cfg, $parms, $type, $module) = @_;
	$cfg->{StyleMap}{$type} = $module;
}

sub AxCacheDir ($$$) {
	my ($cfg, $parms, $cachedir) = @_;
	$cfg->{CacheDir} = $cachedir;
}

sub AxConfigReader ($$$) {
	my ($cfg, $parms, $configclass) = @_;
	$cfg->{ConfigReader} = $configclass;
}

sub AxProvider ($$$) {
	my ($cfg, $parms, $providerclass) = @_;
	$cfg->{Provider} = $providerclass;
}

sub AxStyleProvider ($$$) {
	my ($cfg, $parms, $styleclass) = @_;
	$cfg->{StyleProvider} = $styleclass;
}

sub AxStyle ($$$) {
	my ($cfg, $parms, $style) = @_;
	$cfg->{Style} = $style;
}

sub AxMedia ($$$) {
	my ($cfg, $parms, $media) = @_;
	$cfg->{Media} = $media;
}

sub AxCacheModule ($$$) {
	my ($cfg, $parms, $cachemodule) = @_;
	$cfg->{CacheModule} = $cachemodule;
}

sub AxDebugLevel ($$$) {
	my ($cfg, $parms, $level) = @_;
	$cfg->{DebugLevel} = $level;
}

sub AxOutputCharset ($$$) {
	my ($cfg, $parms, $charset) = @_;
	require Unicode::String;
	require Unicode::Map8;
	$cfg->{OutputCharset} = $charset;
}

# sub AxEncodingDir ($$$) {
# 	my ($cfg, $parms, $dir) = @_;
# 	require Unicode::String;
# 	require Unicode::Map8;
# 	$XML::UM::SlowMapper::ENCDIR = $dir;
# }

sub AxGzipOutput ($$$) {
	my ($cfg, $parms, $arg) = @_;
	$cfg->{GzipOutput} = $arg;
}

################################################
# New mapping directives experiment
################################################

sub AxAddProcessor ($$$$) {
	my ($cfg, $parms, $type, $stylesheet, $media, $style) = @_;
	
	$media ||= 'screen';
	$style ||= '#default';
	
#	warn "AxAddProcessor: $type, $stylesheet [$media, $style]\n";
	
#	warn "Debug Level currently: $cfg->{DebugLevel}\n";
	
	my $processor = ['NORMAL', $type, $stylesheet];
	
	push @{$cfg->{Processors}{$media}{$style}}, $processor;
	return 1;
}

sub AxAddDocTypeProcessor ($$$$$) {
	my ($cfg, $parms, $type, $stylesheet, $pubid, $media, $style) = @_;
	
	$media ||= 'screen';
	$style ||= '#default';
	
#	warn "AxAddDocTypeProcessor: $type, $stylesheet, $pubid [$media, $style]\n";
	
	my $processor = ['DocType', $type, $stylesheet, $pubid];
	
	push @{$cfg->{Processors}{$media}{$style}}, $processor;
}

sub AxAddDTDProcessor ($$$$$) {
	my ($cfg, $parms, $type, $stylesheet, $dtd, $media, $style) = @_;
	
	$media ||= 'screen';
	$style ||= '#default';
	
#	warn "AxAddDTDProcessor: $type, $stylesheet, $dtd [$media, $style]\n";
	
	my $processor = ['DTD', $type, $stylesheet, $dtd];
	
	push @{$cfg->{Processors}{$media}{$style}}, $processor;
}
	
sub AxAddRootProcessor ($$$$$) {
	my ($cfg, $parms, $type, $stylesheet, $root_element, $media, $style) = @_;
	
	$media ||= 'screen';
	$style ||= '#default';
	
#	warn "AxAddRootProcessor: $type, $stylesheet, $root_element [$media, $style]\n";
	
	my $processor = ['Root', $type, $stylesheet, $root_element];
	
	push @{$cfg->{Processors}{$media}{$style}}, $processor;
}

my $ENDMedia = "</AxMediaType>";

sub AxMediaType ($$$;*) {
	my ($cfg, $parms, $media, $cfg_fh, $style) = @_;
	
	$media =~ s/>$//;
	
	$style ||= '#default';
	
#	warn "CFG_FH is a: ", ref($cfg_fh), "\n";
#	warn "Next line is: ", <$cfg_fh>, "\n";
			
#	warn "AxMediaType: $media [$style] [$cfg_fh]\n";
	
	parse_contents($cfg, $parms, $ENDMedia, $cfg_fh, $media, $style);
}

sub AxMediaType_END () {
	die "$ENDMedia outside a <AxMediaType> container\n";
}

my $ENDStyle = "</AxStyleName>";

sub AxStyleName ($$$;*) {
	my ($cfg, $parms, $style, $cfg_fh, $media) = @_;
	
	$style =~ s/>$//;
	
	$media ||= 'screen';
	
#	warn "AxStyleName: $style [$media]\n";
	
	parse_contents($cfg, $parms, $ENDStyle, $cfg_fh, $media, $style);
}

sub AxStyleName_END () {
	die "$ENDStyle outside a <AxStyleName> container\n";
}

sub parse_contents {
	my ($cfg, $parms, $end_token, $cfg_fh, $media, $style) = @_;
	
#	warn "parse_contents ($end_token - $cfg_fh)\n";
	
	while (my $line = <$cfg_fh>) {
#		warn "parse line: $line\n";
		
		last if $line =~ /^$end_token/;
		
		$line =~ s/^(<?\w+)// || die "No command found in $line\n";
		my $cmd = $1;
		
		my @params = parse_params($line);
		
#		warn "Found command: $cmd\n";
		
		no strict 'refs';
		if ($cmd =~ s/^<//) {
			if ($cmd =~ /AxMediaType/) {
				AxMediaType($cfg, $parms, $params[0], $style);
			}
			else {
				AxStyleName($cfg, $parms, $params[0], $media);
			}
		}
		else {
			$cmd->($cfg, $parms, @params, $media, $style);
		}
	}
	return 1;
}

sub parse_params {
	my $line = shift;
	
	my @params;
	
	while ($line =~ /\G\s*((["']).*?\2|\S+)/gc) {
		my $match = $1;
		$match =~ s/^["']//;
		$match =~ s/["']$//;
		push @params, $match;
	}
	
	return @params;
}

sub DIR_MERGE {
	my ($parent, $current) = @_;
	
	my %new;
	
#	warn "DIR MERGE called\n";
	
	# style map
	$new{StyleMap} = { %{$parent->{StyleMap}}, %{$current->{StyleMap}} };
	
	# flat merges (single parameters)
	for my $param (
			qw(CacheDir 
			ConfigReader 
			Provider 
			StyleProvider 
			Style 
			Media 
			CacheModule 
			DebugLevel 
			OutputCharset 
			EncodingDir
			GzipOutput
			)) {
		$new{$param} = $current->{$param} || $parent->{$param};
	}
	
	# merge processor mappings
	
	# from parent
	foreach my $style (keys %{$parent->{Processors}}) {
		foreach my $media (keys %{$parent->{Processors}{$style}}) {
			push @{$new{Processors}{$style}{$media}},
					@{$parent->{Processors}{$style}{$media}};
		}
	}
	
	# from current
	foreach my $style (keys %{$current->{Processors}}) {
		foreach my $media (keys %{$current->{Processors}{$style}}) {
			push @{$new{Processors}{$style}{$media}},
					@{$current->{Processors}{$style}{$media}};
		}
	}
	
	return bless \%new, ref($parent);
}

###############################################################
# AxKit Utility Functions
###############################################################

sub init_debug {
	my $r = shift;
	no strict;
	undef *AxKit::Debug;
	*AxKit::Debug = sub {
		my $level = shift;
		if ($level <= $AxKit::DebugLevel) {
			if ($_[-1] !~ /\n$/) {
				push @_, "\n";
			}
			warn("[AxKit] ", @_);
		}
	};
}

sub reconsecrate {
	my ($object, $class) = @_;
	
	my $module = $class . '.pm';
	$module =~ s/::/\//g;
	
	if (!$INC{$module}) {
		require $module;
	}
	
	bless $object, $class;
}

sub get_subrequest {
	my ($r, $href) = @_;
	
	if ($href =~ /^(http|https|ftp):\/\//i) {
		die "Only relative URI's supported in <?xml-stylesheet?> at this time";
	}
	
	return $r->lookup_uri($href);
}

#########################################
# main mod_perl handler routine
#########################################

sub handler {
	my $r = shift;
	
	Apache::AxKit::Exception->Init();
	
	eval {
		die Apache::AxKit::Exception->Declined(reason => 'in subrequest')
				unless $r->is_main;
		
		die Apache::AxKit::Exception->Declined(reason => 'passthru')
				if $r->notes('axkit_passthru');
		
		$AxKit::Cfg = Apache::AxKit::ConfigReader->new($r);
		my $provider = Apache::AxKit::Provider->new($r);
		
		$AxKit::DebugLevel = $AxKit::Cfg->DebugLevel();
		init_debug($r);
		
		AxKit::Debug(1, "handler called");
	
		# Do we process this URL?
		AxKit::Debug(2, "checking if we process this resource");
		if (!$provider->process()) {
			die Apache::AxKit::Exception->Declined(reason => 'Provider declined');
		}
		
		# get preferred stylesheet and media type
		my $preferred = $AxKit::Cfg->PreferredStyle;
		my $media = $AxKit::Cfg->PreferredMedia;
		
		$preferred ||= '#default';
		
		if ($media !~ /^(screen|tty|tv|projection|handheld|print|braille|aural)$/) {
			$media = 'screen';
		}
		
		AxKit::Debug(2, "media: $media, preferred style: $preferred");

		# get cache object
		my $cache = Apache::AxKit::Cache->new($r, $r->filename(), $preferred, $media);

		my $key = $cache->key();
		
		my $mtime = $provider->mtime();
		
		my ($styles, $ext_ents, $recreate, $reparse);
		
		AxKit::Debug(2, "getting styles and external entities from the XML");
		# get styles/ext_ents from cache or re-parse
		if (exists($AxKit::Stash{$key})
				&& $AxKit::Stash{$key}{mtime} <= $mtime)
		{
			AxKit::Debug(3, "styles and external entities cached");
			($styles, $ext_ents) = 
					@{$AxKit::Stash{$key}}{('styles', 'external_entities')};
		}
		else {
#			warn "No styles in axkit stash\n";
			$recreate++;
			$reparse++;
			
			AxKit::Debug(3, "styles and external entities not cached - calling get_styles()");
			($styles, $ext_ents) = $provider->get_styles($media, $preferred);
			
			$AxKit::Stash{$key} = {
				styles => $styles,
				external_entities => $ext_ents,
				mtime => $mtime,
				};
		}
		
		{
			local $^W;
			if ($preferred && ($styles->[0]{title} ne $preferred)) {
				# we selected a style that didn't exist. 
				# Make sure we default the cache file, otherwise
				# we setup a potential DoS
				$cache = Apache::AxKit::Cache->new($r, $r->filename(), '', $media);
				$key = $cache->key();
			}
		}
		
		if (!$recreate && !$cache->exists()) {
			AxKit::Debug(2, "cache doesn't exist");
#			warn "cache doesn't exist\n";
			$recreate++;
		}
		
		if (!$recreate) {
			my $mtime_cache = $cache->mtime();
			AxKit::Debug(3, "checking stylesheet mtimes vs cache mtime");
			for my $style (@$styles) {
				next unless $style->{href};
				no strict 'refs';
				next unless $style->{module}->stylesheet_exists();
				my $req = get_subrequest($r, $style->{href});
				my $provider = Apache::AxKit::StyleProvider->new($req);
				if (!$provider->exists()) {
					die Apache::AxKit::Exception->Declined(
							reason => "Stylesheet '$style->{href}' does not exist or is not readable"
							);
				}
#				warn "checking $style->{module} mtime against cache: $mtime_cache\n";
				if ($style->{module}->get_mtime($provider)
						<= $mtime_cache)
				{
					AxKit::Debug(3, "stylesheet '$style->{href}' newer than cache");
					$recreate++;
					last;
				}
			}
			
			AxKit::Debug(3, "checking external entities mtimes vs cache mtime");
			# check external entities too now
			if (!$recreate) {
				for my $ext (@$ext_ents) {
					local $^W;
					my $ent = get_subrequest($r, $ext);
					my $ent_provider = Apache::AxKit::Provider->new($ent);
					if ($ent && $ent_provider && ($ent_provider->mtime() <= $mtime_cache)) {
						AxKit::Debug(3, "external entity '$ext' newer than cache");
						$recreate++;
						$reparse++;
						last;
					}
				}
			}
		}
		
		# set default content-type (expat returns in utf-8, so use that)
		$r->content_type('text/html; charset=utf-8');
		
		if (!$recreate) {
			AxKit::Debug(1, "delivering cached copy - all conditions met");
			$cache->deliver();
		}
		
		AxKit::Debug(1, "some condition failed. recreating output");
		
		$AxKit::Cache = $cache;
		
		# reconsecrate Apache request object (& STDOUT) into our own class
		bless $r, 'AxKit::Apache';
		tie *STDOUT, 'AxKit::Apache', $r;

		if (my $charset = $AxKit::Cfg->OutputCharset) {
			AxKit::Debug(5, "Different output charset: $charset");
			$r->content_type("text/html; charset=$charset");
		}
			
		for my $style (@$styles) {
			my $stylereq = get_subrequest($r, $style->{href});
			my $styleprovider = Apache::AxKit::StyleProvider->new($stylereq);

			$r->notes('resetstring', 1);

			no strict 'refs';

			my $mapto = $style->{module};
			
			AxKit::Debug(3, "about to execute: $mapto\::handler");

			my $method = "handler";
			if (defined &{"$mapto\::$method"}) {
				my $retval = $mapto->$method($r, $provider, $styleprovider, $reparse);
			}
			else {
				die Apache::AxKit::Exception->Error(
						text => "$mapto Function not found"
						);
			}
			
			AxKit::Debug(3, "execution of: $mapto\::$method finished");
			
			last if $r->notes('axkit_passthru');
		}
		
		if (my $dom = $r->pnotes('dom_tree')) {
			AxKit::Debug(4, "got a dom_tree back - outputting that");
			$r->notes('resetstring', 1);
			my $output = $dom->toString;
			$dom->dispose();
			$r->print($output);
		}
		
		# restore $r
		bless $r, 'Apache';
		tie *STDOUT, 'Apache', $r;
		
		if (my $charset = $AxKit::Cfg->OutputCharset) {
			my $map = Unicode::Map8->new($charset) || die "Charset: $charset not found in Unicode::Map8 library";
			$r->notes('xml_string', 
					$map->to8( 
						${ Unicode::String::utf8( $r->notes('xml_string') ) } 
					) 
				);
		}
		
		if ($cache->no_cache()) {
			AxKit::Debug(4, "writing xml string to browser");
			if ($AxKit::Cfg->DoGzip) {
				AxKit::Debug(4, 'Sending gzipped xml string to browser');
				$r->send_http_header();
				$r->print(Compress::Zlib::memGzip($r->notes('xml_string')));
			}
			else {
				$r->send_http_header();
				$r->print($r->notes('xml_string'));
			}
		}
		else {
			AxKit::Debug(4, "writing xml string to cache and delivering to browser");
			$cache->write($r->notes('xml_string'));
			
			$cache->deliver();
		}
		
		die Apache::AxKit::Exception->OK();
	};
	if ($@) {
		if (UNIVERSAL::isa($@, 'Apache::AxKit::Exception::Error')) {
			$r->log->error("[AxKit] [Error] $@->{text}");
			return DECLINED;
		}
		elsif (UNIVERSAL::isa($@, 'Apache::AxKit::Exception::Declined')) {
			if ($r->dir_config('AxLogDeclines')) {
				$r->log->info("[AxKit] [DECLINED] $@->{reason}")
						if $@->{reason};
			}
			return DECLINED;
		}
		elsif (UNIVERSAL::isa($@, 'Apache::AxKit::Exception::OK')) {
			return OK;
		}
		else {
			$r->log->error("[AxKit] [UnCaught] $@");
		}
	}
	
	return DECLINED;
}

#########################################################################
# Apache Request Object subclass
#########################################################################

package AxKit::Apache;
use vars qw/@ISA/;
use Apache;
use Fcntl qw(:DEFAULT);
@ISA = ('Apache');

sub TIEHANDLE {
	my($class, $r) = @_;
	$r ||= Apache->request;
}

sub content_type {
	my $self = shift;
	
	my ($type) = @_;

	if ($type && $AxKit::Charset) {
		# don't mess with me, suckaaaa
		unless ($type =~ s/charset=\S+/charset=$AxKit::Charset/) {
			$type .= " charset=$AxKit::Charset";
		}
	}
	
	if ($type && !$AxKit::Cache->no_cache()) {
#		warn "Writing content type '$type'\n";
		my $typecache = Apache::AxKit::Cache->new($self, $AxKit::Cache->key() . '.type');
		$typecache->write($type);
	}
	
	$self->SUPER::content_type(@_);
}

sub print {
	my $self = shift;
	
	if ($self->notes('resetstring')) {
		$self->notes('xml_string', '');
		$self->notes('resetstring', 0);
	}

	$self->notes()->{'xml_string'} .= join('', @_);
}

*PRINT = \&print;

sub no_cache {
	my $self = shift;
	my ($set) = @_;

	$self->SUPER::no_cache(@_);

	if ($set) {
#		warn "caching being turned off!\n";
		$AxKit::Cache->no_cache(1);
	}
}

sub send_http_header {
	my $self = shift;
	my ($content_type) = @_;

	return if $self->notes('headers_sent');

	if ($content_type) {
		$self->content_type($content_type);
	}

	$self->notes('headers_sent', 1);

	$self->SUPER::send_http_header;
}

1;
__END__

=head1 NAME

AxKit - an XML Delivery Toolkit for Apache

=head1 DESCRIPTION

AxKit provides the user with an application development environment
for mod_perl, using XML, Stylesheets and a few other tricks. See 
http://xml.sergeant.org/axkit/ for details.

=head1 SYNOPSIS

In httpd.conf:

	PerlModule AxKit

Then in any Apache configuration section (Files, Location, Directory,
.htaccess):

	# Install AxKit main parts
	SetHandler perl-script
	PerlHandler AxKit
	
	# Setup style type mappings
	AxAddStyleMap text/xsl Apache::AxKit::Language::Sablot
	AxAddStyleMap application/x-xpathscript \
			Apache::AxKit::Language::XPathScript
	
	# Optionally setup a default style mapping
	AxAddDefaultStyleMap /default.xsl text/xsl
	
	# Optionally set a hard coded cache directory
	AxCacheDir /opt/axkit/cachedir
	
Now simply create xml files with stylesheet declarations:

	<?xml version="1.0"?>
	<?xml-stylesheet href="test.xsl" type="text/xsl"?>
	<test>
		This is my test XML file.
	</test>

And for the above, create a stylesheet in the same directory as the
file called "test.xsl" that compiles the XML into something usable  by
the browser. If you wish to use other languages than XSLT, you can,
provided a module exists for that language.

=head1 BUILD PROBLEMS

If you have trouble compiling AxKit, or apache fails to start after 
installing, it's possible to use AxKit without the built in
configuration directives (which have been known to generate segfaults).
To do this install as follows:

	perl Makefile.PL NO_DIRECTIVES=1
	make
	make test
	make install

This removes the custom configuration directives. Note that you may have
to manually remove old AxKit.pm files from your perl library directory
if you have previously built it, because dynamically built libraries
go into the i386 (or whatever processor you have) directory. Now
you can change the directives to ordinary PerlSetVar directives:

	PerlSetVar AxStyleMap "text/xsl => Apache::AxKit::Language::XSLT, \
		application/x-xpathscript => Apache::AxKit::Language::XPathScript"
	
	# note brackets here
	PerlSetVar AxDefaultStyleMap "(/default.xsl text/xsl) \
				(/other.xsl text/xsl)"
	
	PerlSetVar AxCacheDir /opt/axkit/cache
	
It's worth noting that the PerlSetVar option is available regardless of
whether you compile with NO_DIRECTIVES set, although it is marginally
slower to use PerlSetVar.

=cut
