# $Id: ConfigReader.pm,v 1.6 2000/08/06 12:13:42 matt Exp $

package Apache::AxKit::ConfigReader;

use strict;

use Apache::ModuleConfig ();

sub new {
	my $class = shift;
	my $r = shift;
	
	my $cfg = Apache::ModuleConfig->get($r, 'AxKit') || {};
	
	if ($cfg->{ConfigReader} || $r->dir_config('AxConfigReader')) {
		$class = $cfg->{ConfigReader} || $r->dir_config('AxConfigReader');
		my $pkg = $class;
		$pkg =~ s/::/\//g;
		require "$pkg.pm";
	}
	
	return bless { apache => $r, cfg => $cfg }, $class;
}

# returns a hash reference consisting of key = type, value = module
sub StyleMap {
	my $self = shift;
	if ($self->{cfg}->{StyleMap}) {
		return $self->{cfg}->{StyleMap};
	}
	# no StyleMap, try dir_config
	my %hash = split /\s*(?:=>|,)\s*/, $self->{apache}->dir_config('AxStyleMap');
	return \%hash;
}

# returns the location of the cache dir
sub CacheDir {
	my $self = shift;
	if ($self->{cfg}->{CacheDir}) {
		return $self->{cfg}->{CacheDir};
	}
	my $cachedir = $self->{apache}->dir_config('AxCacheDir');
	return $cachedir if $cachedir;
	
	use File::Basename;
	my $dir = dirname($self->{apache}->filename());
	return $dir . "/.xmlstyle_cache";
}

sub ProviderClass {
	my $self = shift;
	if ($self->{cfg}{Provider}) {
		return $self->{cfg}{Provider};
	}
	
	my $provider = $self->{apache}->dir_config('AxProvider');
	return $provider if $provider;
	
	return 'Apache::AxKit::Provider::File';
}

sub StyleProviderClass {
	my $self = shift;
	if ($self->{cfg}{StyleProvider}) {
		return $self->{cfg}{StyleProvider};
	}
	
	my $provider = $self->{apache}->dir_config('AxStyleProvider');
	return $provider if $provider;
	
	return 'Apache::AxKit::Provider::File';
}	

sub PreferredStyle {
	my $self = shift;
	
	return
		$self->{apache}->notes('preferred_style')
				||
		$self->{cfg}->{Style}
				||
		$self->{apache}->dir_config('PreferredStyle')
				||
		'';
}

sub PreferredMedia {
	my $self = shift;
	
	return
		$self->{apache}->notes('preferred_media')
				||
		$self->{cfg}->{Media}
				||
		$self->{apache}->dir_config('PreferredMedia')
				||
		'screen';
}

sub CacheModule {
	my $self = shift;
	return $self->{cfg}{CacheModule}
		|| $self->{apache}->dir_config('CacheModule');
}

sub DebugLevel {
	my $self = shift;
	return $self->{cfg}{DebugLevel} || 
			$self->{apache}->dir_config('DebugLevel') || 
			0;
}

sub OutputCharset {
	my $self = shift;
	return $self->{cfg}{OutputCharset}
		|| $self->{apache}->dir_config('OutputCharset');
}

sub GzipOutput {
	my $self = shift;
	return $self->{cfg}{GzipOutput}
		|| $self->{apache}->dir_config('GzipOutput');
}

sub DoGzip {
	# should I actually send GZip for this request?
	my $self = shift;
	return unless $self->GzipOutput;
	
	AxKit::Debug(5, 'Should we zip the output?');
	my $r = $self->{apache};
	my($can_gzip);
	my @vary = $r->header_out('Vary') if $r->header_out('Vary');
	push @vary, "Accept-Encoding", "User-Agent";
	$r->header_out('Vary',
					join ", ",
					@vary
				);
	my($accept_encoding) = $r->header_in("Accept-Encoding") || '';
	$can_gzip = 1 if index($accept_encoding,"gzip")>=0;
	unless ($can_gzip) {
		my $user_agent = $r->header_in("User-Agent");
		if ($user_agent =~ m{
                    		 ^Mozilla/
                    		 \d+
                    		 \.
                    		 \d+
                    		 [\s\[\]\w\-]+
                    		 (
                    		  \(X11 |
                    		  Macint.+PPC,\sNav
                    		 )
                    		}x
		   ) {
			$can_gzip = 1;
		}
	}

	$r->header_out('Content-Encoding','gzip') if $can_gzip;

	require Compress::Zlib if $can_gzip;
	return $can_gzip;
}

# sub XMLEncodingDir {
# 	my $self = shift;
# 	return $self->{cfg}{EncodingDir}
# 		|| $self->{apache}->dir_config('EncodingDir');
# }

sub GetMatchingProcessors {
	my $self = shift;
	my ($media, $style, $doctype, $dtd, $root) = @_;
	
	my $list = $self->{cfg}{Processors}{$media}{$style};
	
	my @results;
	
	for my $directive (@$list) {
		my $type = $directive->[0];
		if ($type eq 'NORMAL') {
			push @results, { type => $directive->[1], href => $directive->[2] };
		}
		elsif ($type eq 'DocType') {
			if ($doctype eq $directive->[3]) {
				push @results, { type => $directive->[1], href => $directive->[2] };
			}
		}
		elsif ($type eq 'DTD') {
			if ($dtd eq $directive->[3]) {
				push @results, { type => $directive->[1], href => $directive->[2] };
			}
		}
		elsif ($type eq 'Root') {
			if ($root eq $directive->[3]) {
				push @results, { type => $directive->[1], href => $directive->[2] };
			}
		}
		else {
			warn "Unrecognised directive type: $type";
		}
	}
	
	return @results;
}

1;
